import { system, world, EntityEquippableComponent, EquipmentSlot, ItemStack} from "@minecraft/server";import {shields_loop} from './shield.js'
export function getScore(player, objective) {
	return world.scoreboard.getObjective(objective).getScore(player) ?? 0;
}
export function setScore(player, objective, score) {
	return world.scoreboard.getObjective(objective).setScore(player, score) ?? 0;
}
export function addScore(player, objective, score) {
	return world.scoreboard.getObjective(objective).addScore(player, score) ?? 0;
}
const w = world.getDimension("overworld");
world.afterEvents.entitySpawn.subscribe(use =>{
	let e = use.entity;
	if(e.typeId.includes("_wyvern_egg")){
		let nameTag = e.typeId.replace("_wyvern_egg", "")
		e.runCommandAsync("scoreboard objectives add egg_time dummy")
		e.runCommandAsync("scoreboard players set @s egg_time 20")
		e.addTag(`${nameTag}`)
	}
});
let wyverns_type = ["fire", "ice", "ground", "poison", "wither", "ender"]
system.runInterval(() =>{
	try{
		for(let p of world.getPlayers()){
			p.runCommandAsync("gamerule sendcommandfeedback false")
			if(p.hasTag("mount") && p.isJumping){
				p.runCommandAsync("effect @e[family=wyvern,r=4] levitation 1 5 true")
				p.runCommandAsync("event entity @e[family=wyvern,r=4] strs:wyvern_can_fly")
			}
			if(!p.hasTag("mount")){
				p.runCommandAsync("event entity @e[family=wyvern,r=4] strs:wyvern_not_can_fly")
			}
			shields_loop(p)
			item_animations(p)
			for(let types of wyverns_type){
				let wyverns_egg = w.getEntities({type: `strs:${types}_wyvern_egg`})
				for(let we of wyverns_egg){
					let egg_time = getScore(we, "egg_time");
					if(!we.hasTag("shaked")){
						if(egg_time == 10){
							we.runCommandAsync("event entity @s[tag=!shaked] strs:egg_shaked")
							we.runCommandAsync("tag @s[tag=!shaked] add shaked")
						}
					}
					if(we.hasTag("shaked")){
						if(egg_time == 0){
							we.runCommandAsync("event entity @s[tag=shaked] strs:egg_transform")
						}
					}
					let view = p.getEntitiesFromViewDirection({maxDistance: 7});
					view.forEach((look) => {
						let eggTag;
						let firstWord;
						let eggName;
						if(look.entity.typeId == we.typeId){
							look.entity.getTags().forEach(t => {
								if (t.startsWith('strs:')) {
									eggTag = `${t.replace("strs:", "").split(",")}`
									firstWord = t[5].toUpperCase();
									eggName = `${firstWord}`+`${eggTag.replace(`${t[5]}`, "")}`
									p.runCommandAsync(`title @s actionbar §7--${eggName} Wyvern Egg--\n§6Hatching Time: §b${getScore(look.entity, "egg_time")} min`);
								}
							}); 
						}
					});
				}
			}
		}
	}catch(e){

	};
})
system.runInterval(()=>{			
	for(let types of wyverns_type){
		let wyverns_egg = w.getEntities({type: `strs:${types}_wyvern_egg`})
		for(let we of wyverns_egg){
			let egg_time = getScore(we, "egg_time");
			if(egg_time > 0){
				we.runCommandAsync(`scoreboard players remove @s egg_time 1`)
			}
		}
	}
}, 1200)


export function item_animations(p) {
	let equip = p.getComponent(EntityEquippableComponent.componentId);
	try{
		let item = equip?.getEquipment(EquipmentSlot.Mainhand);
		if(item){
			if(item.hasTag("template:shield") && !p.hasTag("mainhand") && !p.hasTag("offhand") && p.isSneaking){
				p.runCommandAsync(`playanimation @s animation.player.custom_shield_block_main_hand root 100`)
				p.addTag("mainhand")
			}
			if(!item.hasTag("template:shield") || p.hasTag("offhand") || !p.isSneaking){
				if(p.hasTag("mainhand")){
					p.runCommandAsync("playanimation @s animation.player.custom_shield_block_main_hand root")
					p.removeTag("mainhand")
				}
			}
		}
		if(!item){
			if(p.hasTag("mainhand")){
				p.runCommandAsync("playanimation @s animation.player.custom_shield_block_main_hand root")
				p.removeTag("mainhand")
			}
		}
	}catch(e){
		
	}
	try{
		let off = equip?.getEquipment(EquipmentSlot.Offhand);
		if(off){
			if(off.hasTag("template:shield")&&p.isSneaking){
				p.addTag("offhand")
				p.runCommandAsync(`playanimation @s animation.player.custom_shield_block_off_hand root 100`)
			}
			if(!off.hasTag("template:shield")||!p.isSneaking){
				if(p.hasTag("offhand")){
					p.runCommandAsync("playanimation @s animation.player.custom_shield_block_off_hand root")
					p.removeTag("offhand")
				}
			}
		}
		if(!off){
			if(p.hasTag("offhand")){
				p.runCommandAsync("playanimation @s animation.player.custom_shield_block_off_hand root")
				p.removeTag("offhand")
			}
		}
	}catch(e){
		
	}
}

world.afterEvents.entityHurt.subscribe(hurt =>{
	try{
		const eHurt = hurt.hurtEntity;
		const damageSource = hurt.damageSource;
		const damage = hurt.damage;
		const cause = damageSource.cause;
		const damaging = damageSource.damagingEntity;
		if(eHurt){
			if(eHurt.typeId == "minecraft:player"&&eHurt.isSneaking){
				const equip = eHurt.getComponent(EntityEquippableComponent.componentId);
				const hand = equip.getEquipment(EquipmentSlot.Mainhand);
				const offhand = equip.getEquipment(EquipmentSlot.Offhand);
				if(hand){
					if(hand.typeId=="new:poison_wyvern_shield"){
						addEffect(damaging, "poison", 30, 1)
					}
					if(hand.typeId=="new:wither_wyvern_shield"){
						addEffect(damaging, "wither", 30, 1)
					}
					if(hand.typeId=="new:ice_wyvern_shield"){
						addEffect(damaging, "slowness", 30, 5)
					}
					if(hand.typeId=="new:fire_wyvern_shield"){
						damaging.setOnFire(5)
					}
					if(hand.typeId=="new:ground_wyvern_shield"){
						damaging.applyDamage(5)
					}
					if(hand.typeId=="new:ender_wyvern_shield"){
						damaging.runCommandAsync("tp ^^7^-3")
						eHurt.playSound("mob.shulker.teleport")
					}
				}
				if(offhand){
					if(offhand.typeId=="new:poison_wyvern_shield"){
						addEffect(damaging, "poison", 30, 1)
					}
					if(offhand.typeId=="new:wither_wyvern_shield"){
						addEffect(damaging, "wither", 30, 1)
					}
					if(offhand.typeId=="new:ice_wyvern_shield"){
						addEffect(damaging, "slowness", 30, 5)
					}
					if(offhand.typeId=="new:fire_wyvern_shield"){
						damaging.setOnFire(5)
					}
					if(offhand.typeId=="new:ground_wyvern_shield"){
						damaging.applyDamage(5)
					}
					if(offhand.typeId=="new:ender_wyvern_shield"){
						damaging.runCommandAsync("tp ^^7^-3")
						eHurt.playSound("mob.shulker.teleport")
					}
				}
			}
		}
	}catch(e){

	};
})