import { world, system, EquipmentSlot, EntityEquippableComponent } from "@minecraft/server"
import { damageItem } from "./lib/durability";

const calculateYDifference = (a, b) => a.y - b.y;
function getDistance3D(p1, p2) {
  return Math.sqrt(
    Math.pow(p2.x - p1.x, 2) +
    Math.pow(p2.y - p1.y, 2) +
    Math.pow(p2.z - p1.z, 2)
  );
}

export function shields_loop(player){
	  const view = player.getViewDirection()
		const loc = player.location
		if (!player.isSneaking) return
		const equipment = player.getComponent("minecraft:equippable");
		const mainHand = equipment?.getEquipmentSlot("Mainhand")?.getItem();
		const offHand = equipment?.getEquipmentSlot("Offhand")?.getItem()
		let shieldHand = null;
		let shieldItem = null;
		if (mainHand?.hasTag("template:shield") && offHand?.typeId !== "minecraft:shield" && !offHand?.hasTag("template:shield")) {
		  shieldHand = "Mainhand";
		  shieldItem = mainHand;
		} 
    else if (offHand?.hasTag("template:shield") && mainHand?.typeId !== "minecraft:shield" && !mainHand?.hasTag("template:shield")) {
		  shieldHand = "Offhand";
		  shieldItem = offHand;
		}
		if (!shieldHand || !shieldItem) return;
		const allEntities = player.dimension.getEntities()
		for (const entity of allEntities) {
		  const isProjectile = entity?.hasComponent("minecraft:projectile")
		  if (isProjectile && getDistance3D({ x: loc.x, y: loc.y + 0.5, z: loc.z }, entity.location) < 2) {
        if (player.getGameMode() === "survival") {
          const newItem = damageItem(shieldItem);
          equipment.setEquipment(shieldHand, newItem);
          world.playSound(newItem ? "item.shield.block" : "random.break", player.location);
        }
        entity.applyImpulse({ x: view.x * 1.5, y: view.y * 1.5, z: view.z * 1.5 });
        entity.remove()
        }
		}
}

world.afterEvents.entityHitEntity.subscribe(({ hitEntity: player, damagingEntity: damager }) => {
  if (player.typeId !== "minecraft:player" || !player.isSneaking) return;
  const health = player.getComponent("minecraft:health");
  const equipment = player.getComponent("minecraft:equippable");
  const mainHand = equipment?.getEquipmentSlot("Mainhand")?.getItem();
  const offHand = equipment?.getEquipmentSlot("Offhand")?.getItem();
  const damagerOnFire = damager.hasComponent("onfire")
  let shieldHand = null;
  let shieldItem = null;
  if (mainHand?.hasTag("template:shield") && offHand?.typeId !== "minecraft:shield" && !offHand?.hasTag("template:shield")) {
    shieldHand = "Mainhand";
    shieldItem = mainHand;
  } else if (offHand?.hasTag("template:shield") && mainHand?.typeId !== "minecraft:shield" && !mainHand?.hasTag("template:shield")) {
    shieldHand = "Offhand";
    shieldItem = offHand;
  }
  if (!shieldHand || !shieldItem) return;
  const rotationDifference = calculateYDifference(player.getRotation(), damager.getRotation());
  if (rotationDifference <= 90 && rotationDifference >= -90) return;
  system.runTimeout(() => {
    if (player.getGameMode() === "survival") {
      const newItem = damageItem(shieldItem);
      equipment.setEquipment(shieldHand, newItem);
      world.playSound(newItem ? "item.shield.block" : "random.break", player.location);
    }
    if (damagerOnFire) player.extinguishFire()
    const damage = player["damage"] ?? 0;
    health.setCurrentValue(health.currentValue + damage);
  });
  const direction = {
    x: damager.location.x - player.location.x,
    y: damager.location.y - player.location.y,
    z: damager.location.z - player.location.z,
  };
  damager.applyKnockback(direction.x, direction.z, 1.3, 0.52);
  player.teleport(player.location, { rotation: player.getRotation() });
});

world.afterEvents.entityHurt.subscribe(({ hurtEntity: player, damage }) => {
  if (player.typeId !== "minecraft:player") return
  player["damage"] = damage
  system.runTimeout(() => player["damage"] = null, 2)
});

world.afterEvents.playerBreakBlock.subscribe(({ player, itemStackAfterBreak: item }) => {
  if (item?.hasTag("template:durability")) {
    const equipment = player.getComponent("minecraft:equippable");
    const newItem = damageItem(item);
    equipment.setEquipment("Mainhand", newItem);
    world.playSound(newItem ? "" : "random.break", player.location);
  }
});

world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
  itemComponentRegistry.registerCustomComponent("template:unbreakable", {
    onBeforeDurabilityDamage: (e) => {
      e.durabilityDamage = 0
    }
  })
})
export function addEffect(player, effectType, duration, level, particles) {
	return player.addEffect(effectType, duration*2, {amplifier: level, showParticles: particles}) ?? 0;
}
export function removeEffect(player, effectType) {
	return player.removeEffect(effectType) ?? 0;
}

world.afterEvents.playerSpawn.subscribe(spawn =>{
	let p = spawn.player;
	if(p.typeId == "minecraft:player"){
		p.runCommandAsync("gamerule sendcommandfeedback false")
	}
})